--
-- mf2170c
-- Class for all mf2170cs
--
-- @author  Felix "Outlaw" S. Clod
-- @date  2/09/09
--
-- Copyright (C) Outlaw, Confidential, All Rights Reserved.

mf2170c = {};

function mf2170c.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Trailer, specializations);
end;

function mf2170c:load(xmlFile)


	self.connectCollisions = SpecializationUtil.callSpecializationsFunction("connectCollisions");
	self.anim = SpecializationUtil.callSpecializationsFunction("anim");
  	self.charId = {};
	self.clipIndex = {};
	self.Go = {}; 
	self.Done = {};
	self.CheckDone = {};
	self.collisionArm = {}; 
	self.moveColli = {}; 
	local count = getXMLInt(xmlFile, "vehicle.animParts#count");
    self.animParts = {}
	local part = self.animParts;
    for i=1, count do
		part[i] = {};
        local partname = string.format("vehicle.animParts.part".."%d", i);
	    local nameR = getXMLString(xmlFile, partname.."#name");
		self.charId[nameR] = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, partname.."#rootNode"));
		self.clipIndex[nameR] = getXMLString(xmlFile, partname.."#animationClip");
		self.CheckDone[nameR] = false;
		
		self.moveColli[nameR] = getXMLString(xmlFile, partname.."#moveColli");
		if self.moveColli[nameR] ~= nil then
			local Collision = {};
			Collision.collision = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, partname.."#collision"));
			Collision.collisionAttacher = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, partname.."#collsionAttacher"));
			Collision.armAttacher = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, partname.. "#armAttacher"));
			Collision.index = 0;
			self.collisionArm[nameR] = Collision;
			self:connectCollisions(nameR);
		end; 
	end; 	
		
		

	local count = getXMLInt(xmlFile, "vehicle.drums#count");
    self.drums = {}
	local part = self.drums;
    for i=1, count do
		part[i] = {};
        local partname = string.format("vehicle.drums.part" .. "%d", i);
	    part[i].index = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, partname .. "#index"));
		local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, partname .. "#rot"));
		part[i].rot = {};
	    part[i].rot[1] = Utils.getNoNil(x, 0);
	    part[i].rot[2] = Utils.getNoNil(y, 0);
	    part[i].rot[3] = Utils.getNoNil(z, 0);
		for j=1, 3 do
			if part[i].rot[j] ~= 0 then
				part[i].rot[j] = 1/part[i].rot[j]
			end;
		end;
    end;
	

    self.getTimeFromLevel = mf2170c.getTimeFromLevel;
    self.moveBales = SpecializationUtil.callSpecializationsFunction("moveBales");
    self.moveBale = SpecializationUtil.callSpecializationsFunction("moveBale");
    self.allowFillType = mf2170c.allowFillType;
    self.fillScale = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.fillScale#value"), 1);

    local firstBaleMarker = getXMLFloat(xmlFile, "vehicle.baleAnimation#firstBaleMarker");
    if firstBaleMarker ~= nil then
        local baleAnimCurve = AnimCurve:new(linearInterpolatorN);
        local keyI = 0;
        while true do
            local key = string.format("vehicle.baleAnimation.key(%d)", keyI);
            local t = getXMLFloat(xmlFile, key.."#time");
            local x,y,z = Utils.getVectorFromString(getXMLString(xmlFile, key.."#pos"));
            if x == nil or y == nil or z == nil then
                break;
            end;
            local rx, ry, rz = Utils.getVectorFromString(getXMLString(xmlFile, key.."#rot"));
            rx = math.rad(Utils.getNoNil(rx, 0));
            ry = math.rad(Utils.getNoNil(ry, 0));
            rz = math.rad(Utils.getNoNil(rz, 0));
            baleAnimCurve:addKeyframe({ v={x, y, z, rx, ry, rz}, time = t});
            keyI = keyI +1;
        end;
        if keyI > 0 then
            self.baleAnimCurve = baleAnimCurve;
            self.baleAnimRoot = Utils.getNoNil(Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.baleAnimation#node")), self.components[1].node);
            self.firstBaleMarker = firstBaleMarker;
            self.baleTypes = {};
            local i = 0;
            while true do
                local key = string.format("vehicle.baleTypes.baleType(%d)", i);
                local t = getXMLString(xmlFile, key.."#fruitType");
                local filename = getXMLString(xmlFile, key.."#filename");
                if t==nil or filename==nil then
                    break;
                end;

                local entry = {};
                entry.filename = filename;
                local desc = FruitUtil.fruitTypes[t];
                if desc ~= nil then
                    self.baleTypes[desc.index] = entry;

                    if self.defaultBaleType == nil then
                        self.defaultBaleType = entry;
                    end;
                end;
                i = i +1;
            end;
            if self.defaultBaleType == nil then
                self.baleTypes = nil;
            end;
        end;
    end;

    local balerSound = getXMLString(xmlFile, "vehicle.balerSound#file");
    if balerSound ~= nil and balerSound ~= "" then
        balerSound = Utils.getFilename(balerSound, self.baseDirectory); 
        self.balerSound = createSample("balerSound");
        self.balerSoundEnabled = false;
        loadSample(self.balerSound, balerSound, false);
        self.balerSoundPitchOffset = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.balerSound#pitchOffset"), 1);
        self.balerSoundVolume = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.balerSound#volume"), 1);
    end;
	
	local airSound = getXMLString(xmlFile, "vehicle.airSound#file");
    if airSound ~= nil and airSound ~= "" then
        airSound = Utils.getFilename(airSound, self.baseDirectory); 
        self.airSound = createSample("airSound");
        self.airSoundEnabled = false;
        loadSample(self.airSound, airSound, false);
        self.airSoundPitchOffset = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.airSound#pitchOffset"), 1);
        self.airSoundVolume = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.airSound#volume"), 1);
    end;
	
		
    self.dustParticleSystems = {};
    local dustParticleSystemCount = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.dustParticleSystems#count"), 0);
    for i=1, dustParticleSystemCount do
        local namei = string.format("vehicle.dustParticleSystems.part%d", i);
		local nodei = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, namei .. "#index"));
        Utils.loadParticleSystem(xmlFile, self.dustParticleSystems, namei, nodei, false, nil, self.baseDirectory)
    end;

	self.keys = {};
    local i=0;
    while true do
        local baseName = string.format("vehicle.keys.input(%d)", i);
        local inputName = getXMLString(xmlFile, baseName.. "#name");
        if inputName == nil then
            break;
        end;
        local inputKey = getXMLString(xmlFile, baseName.. "#key");
        if Input[inputKey] == nil then
            print("Error: invalid key '" .. inputKey .. "'  for input event '" .. inputName .. "'");
            break;
        end;
        self.keys[inputName] = Input[inputKey];
        i = i+1;
    end;	
	
	
	self.counterpath = Utils.getFilename("counter.png", self.baseDirectory);
	self.hudPDAPoxX = 0.435;
    self.hudPDAPoxY = 1-0.330; 
    self.hudPDAWidth = 0.320;
    self.hudPDAHeight = 0.320;
	self.hudBasePoxX = 0.65;
    self.hudBasePoxY = 0.895; 	
    self.hudPDAOverlay = Overlay:new("hudPDAControl", self.counterpath, self.hudPDAPoxX, self.hudPDAPoxY, self.hudPDAWidth, self.hudPDAHeight);
    self.baleMoveLastTime = 0;
	self.counter = 0; 
    self.bales = {};
    self.wasToFast = false;
    self.isTurnedOn = false;
	self.hudActive = true;
	self.checkA = 0;
	self.airC = 0; 
end;




function mf2170c:keyEvent(unicode, sym, modifier, isDown)
	if isDown and sym==self.keys.transport and self.runPickup ~= true then
		self.Go.trsp = not self.Go.trsp; 
		self.Done.trsp = true; 
	
	end;
	
	if isDown and sym==self.keys.pickup and self.Go.trsp == true and self.Go.pickup ~= true and self.CheckDone.trsp == true then
		self.isTurnedOn = not self.isTurnedOn;
		self.runPickup = not self.runPickup;
	end;
	
	if isDown and sym==self.keys.senken and self.runPickup == true and self.CheckDone.trsp == true then
		self.Go.pickup = not self.Go.pickup; 
		self.Done.pickup = true; 
	end;
	
	if isDown and sym==self.keys.PDA then
		self.hudActive = not self.hudActive; 
	end; 
end;


function mf2170c:update(dt)

    self.wasToFast = false;
    if self.attacherVehicle ~= nil then 
	
	
		if self.runPickup == true then
			for i=1, table.getn(self.drums) do
				rotate(self.drums[i].index, self.drums[i].rot[1]  * dt, self.drums[i].rot[2]  * dt, self.drums[i].rot[3]  * dt);
			end;
		end; 		
	
		if self.Go.trsp ~= nil and self.Done.trsp ~= false then
			self:anim("trsp",false); 
		end; 
				
		if self.Go.pickup ~= nil and self.Done.pickup ~= false then			
			self:anim("pickup",false); 
		end; 
		
		if self.Go.pickup == false then
			Utils.setEmittingState(self.dustParticleSystems, false)
			self.checkA = 0; 
		end; 	
		
			
        if self.isTurnedOn and self:allowFillType(FruitUtil.FRUITTYPE_WHEAT)  and self.Go.pickup == true then
            local toFast = self.lastSpeed*3600 > 29;
            if not toFast then
                local totalArea =0;
                local usedFruitType = FruitUtil.FRUITTYPE_UNKNOWN;
                for k, cuttingArea in pairs(self.cuttingAreas) do
                    local x,y,z = getWorldTranslation(cuttingArea.start);
                    local x1,y1,z1 = getWorldTranslation(cuttingArea.width);
                    local x2,y2,z2 = getWorldTranslation(cuttingArea.height);

                    for fruitType,v in pairs(self.fillTypes) do
                        if fruitType ~= FruitUtil.FRUITTYPE_UNKNOWN then
                            local area = Utils.updateFruitWindrowArea(fruitType, x, z, x1, z1, x2, z2, 0);
                            area = area + Utils.updateFruitCutLongArea(fruitType, x, z, x1, z1, x2, z2, 0);
                            if area > 0 then
                                totalArea = totalArea+area;
                                usedFruitType = fruitType;
                            end;
                        end;
                    end;
                end;
				if totalArea == 0 then
					self.checkA = self.checkA + 1;
					if self.checkA >= 500 then
						Utils.setEmittingState(self.dustParticleSystems, false)
						self.checkA = 0; 
					end; 	
				end; 	
                if totalArea > 0 then
                    local literPerPixel = 4*8000/1200 / 6 / (2*2) *12/4;
                    local deltaLevel = totalArea * literPerPixel * self.fillScale;
                    local deltaTime = self:getTimeFromLevel(deltaLevel);
                    self:moveBales(deltaTime);
                    local oldFillLevel = self.fillLevel;
					self:setFillLevel(self.fillLevel+deltaLevel, usedFruitType);
					Utils.setEmittingState(self.dustParticleSystems, true)
                    if self.fillLevel == self.capacity then
						self.counter = self.counter + 1; 
						self.airC = self.airC +1;
						if self.airC == 5 then
							self.airC = 0; 
							playSample(self.airSound, 1, self.airSoundVolume, 0);
						end; 	
                        local restDeltaFillLevel = deltaLevel - (self.fillLevel-oldFillLevel)
                        self:setFillLevel(restDeltaFillLevel, usedFruitType);
                        if self.baleAnimCurve ~= nil and self.baleTypes ~= nil then
                            local baleType = self.baleTypes[usedFruitType];
                            if baleType == nil then
                                baleType = self.defaultBaleType;
                            end;
                            local baleroot = Utils.loadSharedI3DFile(baleType.filename, self.baseDirectory);
                            local baleId = getChildAt(baleroot, 0);
                            setRigidBodyType(baleId, "None");
                            link(self.baleAnimRoot, baleId);
                            delete(baleroot);
                            local bale = {};
                            bale.id = baleId;
                            bale.time = 0;
                            bale.filename = baleType.filename;
                            table.insert(self.bales, bale);
                            self:moveBale(table.getn(self.bales), self:getTimeFromLevel(restDeltaFillLevel));
                        end;
                    end;
                end;
            end;
            self.wasToFast = toFast;
        end;
		
		if not self.balerSoundEnabled and self:getIsActiveForSound() then
            setSamplePitch(self.balerSound, self.balerSoundPitchOffset);
            playSample(self.balerSound, 0, self.balerSoundVolume, 0);
            self.balerSoundEnabled = true;
        end;
			
        if not self.isTurnedOn and self.balerSoundEnabled then
            stopSample(self.balerSound);
			Utils.setEmittingState(self.dustParticleSystems, false)
			self.checkA = 0; 
            self.balerSoundEnabled = false;
        end;
    end;

	
	if self.Go.pillar ~= nil and self.Done.pillar ~= false then
		self:anim("pillar", false); 
	end; 
		
end;


function mf2170c:draw()
    if self.wasToFast then
        g_currentMission:addWarning(g_i18n:getText("Dont_drive_to_fast") .. "\n" .. string.format(g_i18n:getText("Cruise_control_levelN"), "2", InputBinding.getButtonKeyName(InputBinding.SPEED_LEVEL2)), 0.07+0.022, 0.019+0.029);
    end;

	if self.Go.trsp == true and self.Go.pickup ~= true and self.CheckDone.trsp == true then
	    if self.isTurnedOn then	
	        g_currentMission:addExtraPrintText("Taste "..string.upper(string.char(self.keys.pickup))..": "..string.format(self.typeDesc).. " " ..g_i18n:getText("mf2170c_2"));
	    else
	        g_currentMission:addExtraPrintText("Taste "..string.upper(string.char(self.keys.pickup))..": "..string.format(self.typeDesc).. " " ..g_i18n:getText("mf2170c_1"));
	    end;
	end; 
	
	if self.runPickup ~= true  then
		if self.Go.trsp then
			g_currentMission:addExtraPrintText("Taste "..string.upper(string.char(self.keys.transport))..": "..string.format(self.typeDesc).. " " ..g_i18n:getText("mf2170c_3"));
		else
			g_currentMission:addExtraPrintText("Taste "..string.upper(string.char(self.keys.transport))..": "..string.format(self.typeDesc).. " " ..g_i18n:getText("mf2170c_4"));
		end; 	
	end; 	
	
	if self.runPickup == true and self.CheckDone.trsp == true then
		if self.Go.pickup then
			g_currentMission:addExtraPrintText("Taste "..string.upper(string.char(self.keys.senken))..": "..string.format(self.typeDesc).. " " ..g_i18n:getText("mf2170c_6"));
		else
			g_currentMission:addExtraPrintText("Taste "..string.upper(string.char(self.keys.senken))..": "..string.format(self.typeDesc).. " " ..g_i18n:getText("mf2170c_5"));
		end; 	
	end;
	
	
	if self.hudActive then
		g_currentMission:addExtraPrintText("Taste "..string.upper(string.char(self.keys.PDA))..": "..string.format(self.typeDesc).. " " ..g_i18n:getText("mf2170c_8"));
	else
		g_currentMission:addExtraPrintText("Taste "..string.upper(string.char(self.keys.PDA))..": "..string.format(self.typeDesc).. " " ..g_i18n:getText("mf2170c_7"));
	end; 
	
	if self.hudActive == true then
		self.hudPDAOverlay:render();
		setTextBold(true);
		setTextColor(170, 0, 0, 1.0);
		if self.counter >= 10 then
			self.hudBasePoxX = 0.65 - 0.015;
		end; 
		if self.counter >= 100 then
			self.hudBasePoxX = 0.65 - 0.030;
		end; 
		if self.counter >= 1000 then
			self.hudBasePoxX = 0.65 - 0.045;
		end; 
		if self.counter >= 10000 then
			self.hudBasePoxX = 0.65 - 0.060;
		end; 
		if self.counter >= 100000 then
			self.hudBasePoxX = 0.65 - 0.075;
		end; 
		if self.counter >= 1000000 then
			self.hudBasePoxX = 0.65 - 0.090;
		end; 
		renderText(self.hudBasePoxX, self.hudBasePoxY, 0.040, string.format(self.counter));
	end; 	
end;

function mf2170c:onDetach()
	self.Go.pillar = not self.Go.pillar; 
	self.Done.pillar = true; 
	Utils.setEmittingState(self.dustParticleSystems, false)
	self.checkA = 0; 
	if self.balerSoundEnabled then
        stopSample(self.balerSound);
        self.balerSoundEnabled = false;
    end;
    if self.airSoundEnabled then
        stopSample(self.airSound);
        self.airSoundEnabled = false;
    end;	
end;


function mf2170c:onAttach()
	self.Go.pillar = not self.Go.pillar; 
	self.Done.pillar = true; 
end; 



function mf2170c:onLeave()
	if self.balerSoundEnabled then
        stopSample(self.balerSound);
        self.balerSoundEnabled = false;
    end;
    if self.airSoundEnabled then
        stopSample(self.airSound);
        self.airSoundEnabled = false;
    end;	
	Utils.setEmittingState(self.dustParticleSystems, false)
	self.checkA = 0; 
end;






function mf2170c:getTimeFromLevel(level)
    if self.firstBaleMarker ~= nil then
        return level / self.capacity * self.firstBaleMarker;
    end;
    return 0;
end;


function mf2170c:moveBales(dt)
    for i=table.getn(self.bales), 1, -1 do
        self:moveBale(i, dt);
    end;
    self.baleMoveLastTime = self.time
end;


function mf2170c:moveBale(i, dt)
    local bale = self.bales[i];
    bale.time = bale.time + dt;
    local v = self.baleAnimCurve:get(bale.time);
    setTranslation(bale.id, v[1], v[2], v[3]);
    setRotation(bale.id, v[4], v[5], v[6]);
    if bale.time >= 1 then
        local deltaRealTime = (self.time - self.baleMoveLastTime)/1000;
        local lx, ly, lz = bale.lastX, bale.lastY, bale.lastZ;
        local x,y,z = getWorldTranslation(bale.id);
        local rx,ry,rz = getWorldRotation(bale.id);
        link(getRootNode(), bale.id);
        g_currentMission:addItemToSave(bale.filename, bale.id);
        setTranslation(bale.id, x, y, z);
        setRotation(bale.id, rx, ry, rz);
        setRigidBodyType(bale.id, "Dynamic");
        setLinearVelocity(bale.id, (x-lx)/deltaRealTime, (y-ly)/deltaRealTime, (z-lz)/deltaRealTime);
        table.remove(self.bales, i);
        if g_currentMission.baleCount ~= nil then
            g_currentMission.baleCount = g_currentMission.baleCount + 1;
        end;
    else
        bale.lastX, bale.lastY, bale.lastZ = getWorldTranslation(bale.id);
    end;
end;

function mf2170c:allowFillType(fillType)
    return self.fillTypes[fillType] == true;
end;


function mf2170c:anim(varName, loopCheck)
	if self.moveColli[varName] ~= nil then
		local Collision = self.collisionArm[varName];
		setJointFrame(Collision.index, 0, Collision.armAttacher);
	end;
	
	if self.Go[varName] == true and self.Done[varName] ~= false then
		local charId = getAnimCharacterSet(self.charId[varName]);
		local clipIndex = getAnimClipIndex(charId, self.clipIndex[varName]);
		assignAnimTrackClip(charId , 0, clipIndex);
		setAnimTrackLoopState(charId, 0, loopCheck);
		setAnimTrackSpeedScale(charId, 0, 1);
		enableAnimTrack(charId, 0);
		if getAnimTrackTime(charId, 0) >= getAnimClipDuration(charId, clipIndex) and loopCheck == false then
			disableAnimTrack(charId, 0);	
			self.Done[varName] = false;	
			self.CheckDone[varName] = true;	
		end; 
	elseif self.Go[varName] == false and self.Done[varName] ~= false then
		local charId = getAnimCharacterSet(self.charId[varName]);
		local clipIndex = getAnimClipIndex(charId, self.clipIndex[varName]);
		if loopCheck == true then
			disableAnimTrack(charId, 0);
			self.Done[varName] = false;	
		end; 	
		assignAnimTrackClip(charId , 0, clipIndex);
		setAnimTrackLoopState(charId, 0, loopCheck);
		setAnimTrackSpeedScale(charId, 0, -1);
		enableAnimTrack(charId, 0);
		if getAnimTrackTime(charId, 0) <= 0 then
			disableAnimTrack(charId, 0);
			self.Done[varName] = false;	
			self.CheckDone[varName] = false;
		end;
	end;
end; 	



function mf2170c:delete()
	removeJoint(self.collisionArm.pillar.index); 
	removeJoint(self.collisionArm.trsp.index); 
	Utils.setEmittingState(self.dustParticleSystems, false)
	self.checkA = 0; 
    if self.balerSound ~= nil then
        delete(self.balerSound);
    end;
	if self.airSound ~= nil then
        delete(self.airSound);
    end;
end;


function mf2170c:mouseEvent(posX, posY, isDown, isUp, button)
end;


function mf2170c:connectCollisions(varName)
	local Collision = self.collisionArm[varName];
	local constr = JointConstructor:new();
	constr:setActors(self.rootNode, Collision.collision);
	constr:setJointTransforms(Collision.armAttacher, Collision.collisionAttacher);
	for i=1, 3 do		            
		constr:setRotationLimit(i-1, 0, 0, 0);
		constr:setTranslationLimit(i-1, true, 0, 0);
	end;
	Collision.index = constr:finalize();
end;
